@@ -341,7 +341,14 @@ void update_index_for_recovery(ItemPointer tid,
   /* bool isNull; */
   ExprContext *econtext = GetPerTupleExprContext(estate);
   econtext->ecxt_scantuple = lrinode->slot;
-  heap_fetch(reln, SnapshotNow, &htup, &buffer, false, NULL);
+  if (heap_fetch(reln, SnapshotNow, &htup, &buffer, false, NULL) == false)
+    {
+      heap_close(reln, AccessShareLock);      
+#ifdef WAL_DEBUG
+      if (XLOG_DEBUG) elog(LOG, "pgsenna2: tuple failed time qual");
+#endif /* WAL_DEBUG */
+      return;
+    }
 
 #if 0
 #ifdef WAL_DEBUG
