@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 NTT DATA CORPORATION.
+ * Copyright (c) 2006-2008 NTT DATA CORPORATION.
  * All rights reserved.
  */
 
@@ -43,8 +43,8 @@ typedef struct {
   unsigned file_size_keys;
   unsigned nrecords_lexicon;
   unsigned file_size_lexicon;
-  unsigned inv_seg_size;
-  unsigned inv_chunk_size;
+  unsigned long long inv_seg_size;
+  unsigned long long inv_chunk_size;
   void *next_list;
 } sen_index_list;
 
@@ -110,7 +110,7 @@ pgs2destroy(PG_FUNCTION_ARGS)
     do_dir(DataDir, &n);
     closedir(dir);
   } else {
-    elog(ERROR, "pgsenna2: pgs2destroy cannot opendir (%d)", errno);
+    elog(ERROR, "pgsenna2: pgs2destroy cannot opendir (%s)", strerror(errno));
   }
   PG_RETURN_INT32(n);
 }
@@ -153,7 +153,7 @@ do_dir_indexinfo(const char *dirname, int *n, sen_index_list *si_list)
         elog(ERROR, "pgsenna2: pgs2indexinfo cannot open senna index %s",
              idxname);
       }
-      si_list_next = (sen_index_list *)palloc(sizeof(*si_list_next));
+      si_list_next = (sen_index_list *) PGS2_PALLOC(sizeof(*si_list_next));
       sen_index_info(sindex,
                      &(si_list_next->key_size),
                      &(si_list_next->flags),
@@ -211,14 +211,14 @@ _pgs2indexinfo(FunctionCallInfo fcinfo, char *path)
     attinmeta = TupleDescGetAttInMetadata(tupdesc);
     funcctx->attinmeta = attinmeta;
 
-    si_list = (sen_index_list*) palloc(sizeof(*si_list));
+    si_list = (sen_index_list*) PGS2_PALLOC(sizeof(*si_list));
     si_list->next_list = NULL;
     funcctx->user_fctx = (void *)si_list;
     if (dir) {
       do_dir_indexinfo(path, &n, si_list);
       closedir(dir);
     } else {
-      elog(ERROR, "pgsenna2: pgs2indexinfo cannot opendir (%d)", errno);
+      elog(ERROR, "pgsenna2: pgs2indexinfo cannot opendir (%s)", strerror(errno));
     }
     funcctx->max_calls = n;
     MemoryContextSwitchTo(oldcontext);
@@ -238,10 +238,10 @@ _pgs2indexinfo(FunctionCallInfo fcinfo, char *path)
     for (i = 0; i < call_cntr; i++) {
       si_list = si_list->next_list;
     }
-    values = (char **) palloc(12 * sizeof(char *));
-    values[0] = (char *) palloc(MAXPGPATH * sizeof(char));
+    values = (char **) PGS2_PALLOC(12 * sizeof(char *));
+    values[0] = (char *) PGS2_PALLOC(MAXPGPATH * sizeof(char));
     for(i = 1; i < 12; i++) {
-      values[i] = (char *) palloc(24 * sizeof(char));
+      values[i] = (char *) PGS2_PALLOC(24 * sizeof(char));
     }
     snprintf(values[0], MAXPGPATH, "%s", si_list->filename);
     snprintf(values[1], 24, "%d", si_list->dead_flag);
@@ -253,8 +253,8 @@ _pgs2indexinfo(FunctionCallInfo fcinfo, char *path)
     snprintf(values[7], 24, "%u", si_list->file_size_keys);
     snprintf(values[8], 24, "%u", si_list->nrecords_lexicon);
     snprintf(values[9], 24, "%u", si_list->file_size_lexicon);
-    snprintf(values[10], 24, "%u", si_list->inv_seg_size);
-    snprintf(values[11], 24, "%u", si_list->inv_chunk_size);
+    snprintf(values[10], 24, "%llu", si_list->inv_seg_size);
+    snprintf(values[11], 24, "%llu", si_list->inv_chunk_size);
     tuple = BuildTupleFromCStrings(attinmeta, values);
     result = HeapTupleGetDatum(tuple);
     for(i = 0; i < 12; i++) {
@@ -332,14 +332,14 @@ do_dir_getlexicon(const char *dirname, int *n, sym_array **s_array, char *filena
         sen_index *sindex;
         char *keybuf;
         int sid = 1;
-        keybuf = (char *) palloc(sizeof(char *) * SEN_SYM_MAX_KEY_SIZE);
+        keybuf = (char *) PGS2_PALLOC(sizeof(char *) * SEN_SYM_MAX_KEY_SIZE);
         sindex = sen_index_open(idxname);
         if (sindex == NULL) {
           elog(ERROR, "pgsenna2: pgs2getlexicon cannot open senna index %s",
                idxname);
         }
         (*n) = 0;
-        s_array = (sym_array **) palloc(sen_sym_size(sindex->lexicon) *
+        s_array = (sym_array **) PGS2_PALLOC(sen_sym_size(sindex->lexicon) *
                                         sizeof(**s_array));
         while (true) {
           keybuf[0] = '\0';
@@ -347,10 +347,10 @@ do_dir_getlexicon(const char *dirname, int *n, sym_array **s_array, char *filena
           if (sid == SEN_SYM_NIL) {
             break;
           }
-          s_array[*n] = (sym_array *) palloc(sizeof(*s_array[*n]));
+          s_array[*n] = (sym_array *) PGS2_PALLOC(sizeof(*s_array[*n]));
           s_array[*n]->id = sid;
-          s_array[*n]->lexicon_array = (char *) palloc(strlen(keybuf) *
-                                                       sizeof(char));
+          s_array[*n]->lexicon_array = (char *) PGS2_PALLOC(strlen(keybuf) *
+                                                            sizeof(char));
           if(sen_inv_cursor_open(sindex->inv , sid, 1)) {
             s_array[*n]->flag = 1;
           } else {
@@ -408,7 +408,7 @@ pgs2getlexicon(PG_FUNCTION_ARGS)
 #endif
       closedir(dir);
     } else {
-      elog(ERROR, "pgsenna2: pgs2getlexicon cannot opendir (%d)", errno);
+      elog(ERROR, "pgsenna2: pgs2getlexicon cannot opendir (%s)", strerror(errno));
     }
     funcctx->user_fctx = s_array;
     funcctx->max_calls = n;
@@ -427,10 +427,10 @@ pgs2getlexicon(PG_FUNCTION_ARGS)
     int len;
 
     len = strlen(s_array[call_cntr]->lexicon_array) + 1;
-    values = (char **) palloc(3 * sizeof(char *));
-    values[0] = (char *) palloc(16 * sizeof(char));
-    values[1] = (char *) palloc(len * sizeof(char));
-    values[2] = (char *) palloc(16 * sizeof(char));
+    values = (char **) PGS2_PALLOC(3 * sizeof(char *));
+    values[0] = (char *) PGS2_PALLOC(16 * sizeof(char));
+    values[1] = (char *) PGS2_PALLOC(len * sizeof(char));
+    values[2] = (char *) PGS2_PALLOC(16 * sizeof(char));
     snprintf(values[0], 16, "%d", s_array[call_cntr]->id);
     snprintf(values[1], len, "%s", s_array[call_cntr]->lexicon_array);
     snprintf(values[2], 16, "%d", s_array[call_cntr]->flag);
@@ -452,7 +452,7 @@ pgs2version(PG_FUNCTION_ARGS)
 {
   text *buffer;
 
-  buffer = (text *) palloc(VARHDRSZ + strlen(PGS2_PACKAGE_STRING));
+  buffer = (text *) PGS2_PALLOC(VARHDRSZ + strlen(PGS2_PACKAGE_STRING));
 #ifdef POSTGRES83
   SET_VARSIZE_4B(buffer, VARHDRSZ + strlen(PGS2_PACKAGE_STRING));
 #else
@@ -477,9 +477,9 @@ pgs2seninfo(PG_FUNCTION_ARGS)
   unsigned int initial_n_segments;
   unsigned int partial_match_threshold;
 
-  version = (char *) palloc(24);
-  configure_options = (char *) palloc(256);
-  config_path = (char *) palloc(MAXPGPATH);
+  version = (char *) PGS2_PALLOC(24);
+  configure_options = (char *) PGS2_PALLOC(256);
+  config_path = (char *) PGS2_PALLOC(MAXPGPATH);
   sen_info(&version, &configure_options, &config_path,
            &default_encoding, &initial_n_segments, &partial_match_threshold);
   if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE) {
@@ -489,17 +489,17 @@ pgs2seninfo(PG_FUNCTION_ARGS)
                     "that cannot accept type record")));
   }
   attinmeta = TupleDescGetAttInMetadata(tupdesc);
-  values[0] = palloc(24);
+  values[0] = PGS2_PALLOC(24);
   snprintf(values[0], 24, "%s", version);
-  values[1] = palloc(256);
+  values[1] = PGS2_PALLOC(256);
   snprintf(values[1], 256, "%s", configure_options);
-  values[2] = palloc(MAXPGPATH);
+  values[2] = PGS2_PALLOC(MAXPGPATH);
   snprintf(values[2], MAXPGPATH, "%s", config_path);
-  values[3] = palloc(24);
+  values[3] = PGS2_PALLOC(24);
   snprintf(values[3], 24, "%d", default_encoding);
-  values[4] = palloc(24);
+  values[4] = PGS2_PALLOC(24);
   snprintf(values[4], 24, "%d", initial_n_segments);
-  values[5] = palloc(24);
+  values[5] = PGS2_PALLOC(24);
   snprintf(values[5], 24, "%d", partial_match_threshold);
   result = HeapTupleGetDatum(BuildTupleFromCStrings(attinmeta, values));
   PG_RETURN_DATUM(result);
@@ -574,17 +574,17 @@ pgs2getoption(PG_FUNCTION_ARGS)
                     "that cannot accept type record")));
   }
   attinmeta = TupleDescGetAttInMetadata(tupdesc);
-  values[0] = palloc(16);
+  values[0] = PGS2_PALLOC(16);
   snprintf(values[0], 16, "%d", max_n_sort_result);
-  values[1] = palloc(16);
+  values[1] = PGS2_PALLOC(16);
   snprintf(values[1], 16, "%s", enable_seqscan);
-  values[2] = palloc(16);
+  values[2] = PGS2_PALLOC(16);
   snprintf(values[2], 16, "%d", seqscan_flags);
-  values[3] = palloc(16);
+  values[3] = PGS2_PALLOC(16);
   snprintf(values[3], 16, "%d", sen_index_flags);
-  values[4] = palloc(16);
+  values[4] = PGS2_PALLOC(16);
   snprintf(values[4], 16, "%d", max_n_index_cache);
-  values[5] = palloc(16);
+  values[5] = PGS2_PALLOC(16);
   snprintf(values[5], 16, "%d", initial_n_segments);
   result = HeapTupleGetDatum(BuildTupleFromCStrings(attinmeta, values));
   PG_RETURN_DATUM(result);
