@@ -967,7 +967,9 @@ pgs2rescan(PG_FUNCTION_ARGS)
   if (!r) {
     elog(ERROR, "pgsenna2: sen_records_open failed");
   }
+#if 0
   r->ignore_deleted_records = s->ignore_killed_tuples ? 1 : 0;
+#endif /* 0 */
   for (i = 0, op = sen_sel_or; i < s->numberOfKeys; i++, op = sen_sel_and) {
     elog(DEBUG1,
          "pgsenna2: sk_flags=%d,atn=%d,strategy=%d,subtype=%d,argument=%p",
@@ -1045,6 +1047,7 @@ pgs2endscan(PG_FUNCTION_ARGS)
 Datum
 pgs2markpos(PG_FUNCTION_ARGS)
 {
+#if 0
   IndexScanDesc s = (IndexScanDesc) PG_GETARG_POINTER(0);
   scan_stat *ss = (scan_stat *) s->opaque;
 
@@ -1052,7 +1055,8 @@ pgs2markpos(PG_FUNCTION_ARGS)
   ss->mark = ss->records->curr_rec;
 #ifndef POSTGRES83
   s->currentMarkData = s->currentItemData;
-#endif
+#endif /* POSTGRES83 */
+#endif /* 0 */
   PG_RETURN_VOID();
 }
 
@@ -1060,6 +1064,7 @@ pgs2markpos(PG_FUNCTION_ARGS)
 Datum
 pgs2restrpos(PG_FUNCTION_ARGS)
 {
+#if 0
   IndexScanDesc s = (IndexScanDesc) PG_GETARG_POINTER(0);
   scan_stat *ss = (scan_stat *) s->opaque;
 
@@ -1067,7 +1072,9 @@ pgs2restrpos(PG_FUNCTION_ARGS)
   ss->records->curr_rec = ss->mark;
 #ifndef POSTGRES83
   s->currentItemData = s->currentMarkData;
-#endif
+#endif /* POSTGRES83 */
+#endif /* 0 */
+  elog(ERROR, "pgsenna2: Ludia doesn't support Merge-Join");
   PG_RETURN_VOID();
 }
 
@@ -1243,16 +1250,17 @@ pgs2contain0(Datum vt, Datum vq, Oid oid_type)
     ct = (char **) PGS2_PALLOC(sizeof(char *));
     *ct = datum2cstr(vt);
   } else {
+    bits8 *bitmap = NULL;
+    int bitmask = 1;
     ArrayType* vttype = DatumGetArrayTypeP(vt);
     char* v_ptr = ARR_DATA_PTR(vttype);
     ct_strlen_n = *ARR_DIMS(vttype);
     ct = (char **) PGS2_PALLOC(ct_strlen_n * sizeof(char *));
 #ifdef POSTGRES81
-    bits8 *bitmap = NULL;
+    bitmap = NULL;
 #else
-    bits8 *bitmap = ARR_NULLBITMAP(vttype);
+    bitmap = ARR_NULLBITMAP(vttype);
 #endif
-    int bitmask = 1;
     for(i = 0  ; i < ct_strlen_n; i++) {
       if (!bitmap || (*bitmap & bitmask)) {
         ct[i] = datum2cstr(fetch_att(v_ptr, false, -1));
